
/**
 * wavfilewriter: Helper class to write .wav file headers
 * Copyright (C) 2016 Thomas Perl <m@thp.io>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 **/

#ifndef _WAVEFILEWRITER_H
#define _WAVEFILEWRITER_H

#include <QString>
#include <QFile>

#include <stdint.h>

class WaveFileWriter {
public:
    WaveFileWriter();
    ~WaveFileWriter();

    bool open(const QString &filename, int sampleRate, int sampleSize, int channels);
    bool write_signed_big_endian(const int16_t *data, size_t samples);
    void close();

private:
    QFile output;
    bool updateSize;
};

#endif /* _WAVEFILEWRITER_H */
